@@ -2,7 +2,6 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-import numpy as np
 import pytest
 
 import narwhals as nw
@@ -54,9 +53,11 @@ def test_no_arg_when_fail(constructor: Constructor) -> None:
 
 
 def test_value_numpy_array(constructor_eager: ConstructorEager) -> None:
-    df = nw.from_native(constructor_eager(data))
+    pytest.importorskip("numpy")
     import numpy as np
 
+    df = nw.from_native(constructor_eager(data))
+
     result = df.select(nw.when(nw.col("a") == 1).then(np.arange(3, 6)).alias("a_when"))
     expected = {"a_when": [3, None, None]}
     assert_equal_data(result, expected)
@@ -80,6 +81,9 @@ def test_value_expression(constructor: Constructor) -> None:
 
 
 def test_otherwise_numpy_array(constructor_eager: ConstructorEager) -> None:
+    pytest.importorskip("numpy")
+    import numpy as np
+
     df = nw.from_native(constructor_eager(data))
 
     arr: _1DArray = np.zeros([3], np.dtype(np.int64))
