@@ -7,8 +7,6 @@ from contextlib import nullcontext as does_not_raise
 from datetime import datetime, timedelta
 from typing import TYPE_CHECKING, Any, Callable, cast
 
-import numpy as np
-import pandas as pd
 import pytest
 
 import narwhals as nw
@@ -117,7 +115,11 @@ def test_when_then() -> None:
 
 
 def test_constructors() -> None:
+    pytest.importorskip("pandas")
     pytest.importorskip("pyarrow")
+    import numpy as np
+    import pandas as pd
+
     if PANDAS_VERSION < (2, 2):
         pytest.skip()
     assert nw_v1.new_series("a", [1, 2, 3], backend="pandas").to_list() == [1, 2, 3]
@@ -265,6 +267,9 @@ def test_is_ordered_categorical_interchange_protocol() -> None:
 
 
 def test_all_nulls_pandas() -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     assert (
         nw_v1.from_native(pd.Series([None] * 3, dtype="object"), series_only=True).dtype
         == nw_v1.Object
@@ -272,6 +277,9 @@ def test_all_nulls_pandas() -> None:
 
 
 def test_int_select_pandas() -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     df = nw_v1.from_native(pd.DataFrame({0: [1, 2], "b": [3, 4]}))
     with pytest.raises(
         nw_v1.exceptions.InvalidIntoExprError, match="\n\nHint:\n- if you were trying"
@@ -315,6 +323,9 @@ def test_cast_to_enum_v1(
 
 
 def test_v1_ordered_categorical_pandas() -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     s = nw_v1.from_native(
         pd.Series([0, 1], dtype=pd.CategoricalDtype(ordered=True)), series_only=True
     )
@@ -360,6 +371,9 @@ def test_v1_enum_duckdb_2550() -> None:
     ],
 )
 def test_is_native_dataframe(is_native_dataframe: Callable[[Any], Any]) -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     data = {"a": [1, 2], "b": ["bar", "foo"]}
     df = nw.from_native(pd.DataFrame(data))
     assert not is_native_dataframe(df)
@@ -377,6 +391,9 @@ def test_is_native_dataframe(is_native_dataframe: Callable[[Any], Any]) -> None:
     ],
 )
 def test_is_native_series(is_native_series: Callable[[Any], Any]) -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     data = {"a": [1, 2]}
     ser = nw.from_native(pd.DataFrame(data))["a"]
     assert not is_native_series(ser)
@@ -505,6 +522,9 @@ def test_renamed_taxicab_norm_dataframe_narwhalify() -> None:
 
 
 def test_dtypes() -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     df = nw_v1.from_native(
         pd.DataFrame({"a": [1], "b": [datetime(2020, 1, 1)], "c": [timedelta(1)]})
     )
@@ -530,6 +550,9 @@ def test_dtypes() -> None:
     ],
 )
 def test_strict(strict: Any, context: Any) -> None:
+    pytest.importorskip("numpy")
+    import numpy as np
+
     arr = np.array([1, 2, 3])
 
     with context:
@@ -875,6 +898,7 @@ def test_gather_every(constructor_eager: ConstructorEager, n: int, offset: int)
 def test_gather_every_dask_v1(n: int, offset: int) -> None:
     pytest.importorskip("dask")
     import dask.dataframe as dd
+    import pandas as pd
 
     data = {"a": list(range(10))}
 
@@ -903,6 +927,9 @@ def test_unique_series_v1() -> None:
 
 
 def test_invalid() -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     df = nw.from_native(pd.DataFrame({"a": [1, 2]}))
     with pytest.raises(InvalidOperationError):
         df.select(nw_v1.col("a").mean().head())
@@ -911,6 +938,9 @@ def test_invalid() -> None:
 
 
 def test_deprecated_expr_methods() -> None:
+    pytest.importorskip("pandas")
+    import pandas as pd
+
     data = {"a": [0, 0, 2, -1]}
     df = nw_v1.from_native(pd.DataFrame(data), eager_only=True)
     result = df.select(
@@ -937,6 +967,7 @@ def test_dask_order_dependent_ops() -> None:
     # raise after stable.v1.
     pytest.importorskip("dask")
     import dask.dataframe as dd
+    import pandas as pd
 
     df = nw_v1.from_native(dd.from_pandas(pd.DataFrame({"a": [1, 2, 3]})))
     result = df.select(
@@ -1029,6 +1060,9 @@ def test_dataframe_from_arrow(eager_backend: EagerAllowed) -> None:
 
 
 def test_dataframe_from_numpy(eager_backend: EagerAllowed) -> None:
+    pytest.importorskip("numpy")
+    import numpy as np
+
     arr: _2DArray = cast("_2DArray", np.array([[5, 2, 0, 1], [1, 4, 7, 8], [1, 2, 3, 9]]))
     schema = {"c": nw.Int16(), "d": nw.Float32(), "e": nw.Int16(), "f": nw.Float64()}
     expected = {"c": [5, 1, 1], "d": [2, 4, 2], "e": [0, 7, 3], "f": [1, 8, 9]}
@@ -1060,6 +1094,9 @@ def test_dataframe_from_numpy(eager_backend: EagerAllowed) -> None:
 def test_series_from_numpy(
     eager_backend: EagerAllowed, dtype: IntoDType | None, expected: Sequence[Any]
 ) -> None:
+    pytest.importorskip("numpy")
+    import numpy as np
+
     arr: _1DArray = cast("_1DArray", np.array([5, 2, 0, 1]))
     name = "abc"
     result = nw_v1.Series.from_numpy(name, arr, backend=eager_backend, dtype=dtype)
